% test NL means denoising
%
%   Copyright (c) 2007 Gabriel Peyre

clear all;

path(path,'toolbox/');
path(path, 'images/');

%% load the image
%name = 'lenacoul';
name = 'barb';
M = load_image(name);
m = size(M,1); n = size(M,2);

% crop the image
%M = crop(M, [m n]);

sigma = 25; % variance of additional noise
% avoid saturation
M0 = M;

RandStream.setDefaultStream(RandStream('mt19937ar','seed',100));
M = M0 + sigma * randn(size(M));

%% options of NL means
options.k = 2;          % half size for the windows
options.T = 14.91;      % width of the gaussian, relative to max(M(:))  (=1 here)
options.max_dist = 20;  % search width, the smaller the faster the algorithm will be
options.ndims = Inf;    % number of dimension used for distance computation (PCA dim.reduc. to speed up)
options.do_patchwise = 0;

%% do denoising
tic;
[M1,Wx,Wy,options,Mc,H1,div] = perform_nl_means(M, options);
toc;

%% compute and report SURE
orig_mse=mean(abs(M(:)-M0(:)).^2);
true_mse=norm(M1(:)-M0(:))^2/(m*n);
sure_mse=norm(M(:)-M1(:))^2/(m*n) - sigma^2 + 2*sigma^2/(m*n)*sum(div(:));

psnr_orig=-10*log10(orig_mse/255^2);
psnr_true=-10*log10(true_mse/255^2);
psnr_sure=-10*log10(sure_mse/255^2);

fprintf('PSNR noisy image: %3.1f, PSNR denoised: %3.1f, PSNR-SURE denoised: %3.1f\n',psnr_orig,psnr_true,psnr_sure);

clf;
imagesc(M1);
colormap gray(256);
return;

%% display results
ax = [];
clf;
ax(1) = subplot(2,2,1);
imagesc(M); axis image; axis off;
title('Original image');
ax(2) = subplot(2,2,2);
imagesc(clamp(M1)); axis image; axis off;
title('Denoised');
ax(3) = subplot(2,2,3);
imagesc(rescale(M-M1));
title('Removed noise');
axis image; axis off;
if size(M,3)>1
    colormap gray(256);
end
linkaxes(ax,'xy');